@@ -78,7 +78,7 @@ class LayoutDependency(flow.Object):
             asset_type = self.asset_type.get()
             asset = None
 
-            if self.name() in ["audio", "board"]:
+            if self.name().startswith("audio") or self.name() == "board":
                 asset = self._parent._shot
             else:
                 asset_family = self.asset_family.get()
@@ -178,7 +178,7 @@ class LayoutDependencies(flow.DynamicMap):
             )
         
         # Audio file and storyboard
-        for asset_name in ['audio', 'board']:
+        for asset_name in ['audio', 'audio_voices', 'audio_effects', 'board']:
             casting[asset_name] = dict(
                 asset_type=asset_name,
                 asset_family='misc',
@@ -202,13 +202,15 @@ class LayoutDependencies(flow.DynamicMap):
             "chars": "rig_ok_blend",
             "props": "rig_ok_blend",
             "audio": "audio_wav",
+            "audio_voices": "audio_voices_wav",
+            "audio_effects": "audio_effects_wav",
             "board": "board_mp4",
         }[asset_type]
     
     def files_from_asset_type(self, asset, asset_type):
         if asset_type == "sets":
             return asset.departments.design.files
-        elif asset_type in ["audio", "board"]:
+        elif asset_type.startswith('audio') or asset_type == 'board':
             return asset.departments.misc.files
         else:
             return asset.departments.rigging.files
@@ -244,7 +246,7 @@ class LayoutDependencies(flow.DynamicMap):
         style["icon"] = icon_by_status[item.available.get()]
 
 
-class BuildBlenderScene(RunAction):
+class BuildBlenderScene(baseflow.file.GenericRunAction):
 
     _department = flow.Parent()
     _shot = flow.Parent(3)
@@ -258,6 +260,17 @@ class BuildBlenderScene(RunAction):
     def get_run_label(self):
         return 'Build scene'
     
+    def get_version(self, button):
+        # Make build action behave as base RunAction by default
+        return RunAction.get_version(self, button)
+
+    def get_buttons(self):
+        # Make build action behave as base RunAction by default
+        return RunAction.get_buttons(self)
+
+    def needs_dialog(self):
+        return True
+    
     def allow_context(self, context):
         return context and context.endswith(".inline")
 
@@ -289,6 +302,24 @@ class BuildBlenderScene(RunAction):
 
         return revision.get_path()
 
+    def _ensure_folder(self, name, to_edit=False, publish_comment=""):
+        files = self._department.files
+
+        if not files.has_mapped_name(name):
+            files.create_folder.folder_name.set(name)
+            files.create_folder.run(None)
+
+        folder = files[name]
+
+        if to_edit:
+            revision = folder.create_working_copy()
+        else:
+            revision = folder.publish(comment=publish_comment)
+
+        folder.set_current_user_on_revision(revision.name())
+
+        return revision.get_path()
+
     def _blender_cmd(self, operator, **kwargs):
         '''
         Returns Blender scene builder operator command as a string.
@@ -296,7 +327,8 @@ class BuildBlenderScene(RunAction):
         Operator must be one of the following: `setup`, `setup_anim`,
                                                `add_asset`, `add_set`, `add_audio`, `add_board`,
                                                `update_audio", "update_storyboard`,
-                                               `remap_relative`, `save`.
+                                               `export_ae`,
+                                               `cleanup`, `save`.
         '''
 
         blender_operators = {
@@ -315,11 +347,14 @@ class BuildBlenderScene(RunAction):
                           'args': 'filepath="{filepath}"'},
 
             "update_audio": {'operator_command': 'bpy.ops.pipeline.scene_builder_update_audio',
-                             'args' :'filepath="{filepath}"'},
+                             'args' :''},
             "update_board": {'operator_command': 'bpy.ops.pipeline.scene_builder_update_storyboard',
                              'args' :'filepath="{filepath}"'},
 
-            "remap_relative": {'operator_command': 'bpy.ops.file.make_paths_relative',
+            "export_ae": {'operator_command': 'bpy.ops.pipeline.scene_builder_export_ae',
+                          'args' :'filepath="{filepath}"'},
+
+            "cleanup": {'operator_command': 'bpy.ops.pipeline.scene_builder_cleanup',
                                'args': ''},
             "save": {'operator_command': 'bpy.ops.wm.save_mainfile',
                      'args': 'filepath="{filepath}", compress=True'},
@@ -347,7 +382,7 @@ class BuildLayoutScene(BuildBlenderScene):
 
         assets = self._shot_data.get("assets_data", [])
         sets = self._shot_data.get("sets_data", [])
-        audio_path = self._shot_data.get("audio_path", None)
+        audio_paths = self._shot_data.get("audio_paths", [])
         board_path = self._shot_data.get("board_path", None)
         layout_path = self._shot_data["layout_path"] # Mandatory
         template_path = resources.get("file_templates", "template.blend")
@@ -362,11 +397,13 @@ class BuildLayoutScene(BuildBlenderScene):
                 python_expr += self._blender_cmd("add_asset", filepath=path, asset_name=name, asset_type=asset_type)
         for set_dir, set_dicts in sets:
             python_expr += self._blender_cmd("add_set", set_dir=set_dir, set_dicts=set_dicts)
-
-        if audio_path: python_expr += self._blender_cmd("add_audio", filepath=audio_path)
+        
+        for audio_path in audio_paths:
+            python_expr += self._blender_cmd("add_audio", filepath=audio_path)
+        
         if board_path: python_expr += self._blender_cmd("add_board", filepath=board_path)
 
-        python_expr += self._blender_cmd("remap_relative")
+        # python_expr += self._blender_cmd("cleanup")
         python_expr += self._blender_cmd("save", filepath=layout_path)
 
         return [
@@ -412,6 +449,7 @@ class BuildLayoutScene(BuildBlenderScene):
         self._shot_data["nb_frames"] = shot_data["nb_frames"]
         self._shot_data["assets_data"] = []
         self._shot_data["sets_data"] = []
+        self._shot_data["audio_paths"] = []
 
         for dep in self.dependencies.mapped_items():
             if dep.available.get() != "Available":
@@ -429,8 +467,10 @@ class BuildLayoutScene(BuildBlenderScene):
                 self._shot_data["sets_data"].append(
                     (path, [{"name": name} for name in set_names])
                 )
-            else:
-                self._shot_data["%s_path" % asset_type] = path
+            elif asset_type == "board":
+                self._shot_data["board_path"] = path
+            elif asset_type.startswith("audio"):
+                self._shot_data["audio_paths"].append(path)
 
         # Configure layout file
         layout_path = self._ensure_file(
@@ -478,7 +518,7 @@ class BuildBlockingScene(BuildBlenderScene):
                 assets.append({"name": name,
                                 "filepath": path,
                                 "target_collection": asset_type})
-        audio_path = self._shot_data.get("audio_path", None)
+        
         board_path = self._shot_data.get("board_path", None)
 
         # Build Blender Python expression
@@ -488,12 +528,12 @@ class BuildBlockingScene(BuildBlenderScene):
 
         # Update reference files
 
-        if audio_path:
-            python_expr += self._blender_cmd("update_audio", filepath=audio_path)
+        python_expr += self._blender_cmd("update_audio")
+        
         if board_path:
             python_expr += self._blender_cmd("update_board", filepath=board_path)
 
-        python_expr += self._blender_cmd("remap_relative")
+        python_expr += self._blender_cmd("cleanup")
         python_expr += self._blender_cmd("save", filepath=anim_path)
 
         return [
@@ -514,7 +554,7 @@ class BuildBlockingScene(BuildBlenderScene):
             buttons =  ["Cancel"]
         else:
             msg = "<h2>Configure animation shot</h2>"
-            buttons = ["Build and edit", "Build and publish", "Cancel"]
+            buttons = ["Build and edit", "Cancel"]
 
         self.message.set(msg)
 
@@ -545,8 +585,10 @@ class BuildBlockingScene(BuildBlenderScene):
 
             if asset_type in ["chars", "props"]:
                 self._shot_data["assets_data"].append((dep.name(), path, asset_type + '_', asset_number))
-            else:
-                self._shot_data["%s_path" % asset_type] = path
+            elif asset_type == "board":
+                self._shot_data["board_path"] = path
+            # Ignore audio files here, as they will be updated from the ones
+            # already present in the original layout scene
 
         # Configure anim file
         latest_revision = self._shot.departments.layout.files["layout_blend"].get_head_revision()
@@ -556,7 +598,7 @@ class BuildBlockingScene(BuildBlenderScene):
         anim_path = self._ensure_file(
             name='blocking',
             format='blend',
-            to_edit=(button == 'Build and edit'),
+            to_edit=True,
             src_path=layout_path,
             publish_comment="Created with anim scene builder"
         )
@@ -673,9 +715,114 @@ class AnimationDepartment(Department):
     create_anim_scene = flow.Child(CreateAnimationScene)
 
 
+class BuildRenderScene(BuildBlenderScene):
+
+    _department = flow.Parent()
+    _shot = flow.Parent(3)
+    _sequence = flow.Parent(5)
+
+    def extra_argv(self):
+        # Get scene builder arguments
+        render_path = self._shot_data.get("render_path", None)
+        jsx_path = self._shot_data.get("jsx_path", None)
+
+        # Build Blender Python expression
+        python_expr = "import bpy\n"
+        python_expr += self._blender_cmd("cleanup")
+        python_expr += self._blender_cmd("export_ae", filepath=jsx_path)
+
+        return [
+            "-b", render_path,
+            "--addons", "io_export_after_effects,lfs_scene_builder",
+            "--python-expr", wrap_python_expr(python_expr)
+        ]
+
+    def get_buttons(self):
+        latest_revision = None
+        files = self._shot.departments.animation.files
+
+        if "animation_blend" in files.mapped_names():
+            latest_revision = files["animation_blend"].get_head_revision()
+        elif "blocking_blend" in files.mapped_names():
+            latest_revision = files["blocking_blend"].get_head_revision()
+        elif "breakdown_blend" in files.mapped_names():
+            latest_revision = files["breakdown_blend"].get_head_revision()
+
+        if (latest_revision is None or latest_revision.get_sync_status() != 'Available'):
+            msg = "<h2><font color=#D5000D>Last revision of animation file not available</font></h2>"
+            buttons =  ["Cancel"]
+        else:
+            msg = "<h2>Configure render shot</h2>"
+            buttons = ["Build and edit", "Cancel"]
+            self.latest_revision = latest_revision
+
+        self.message.set(msg)
+
+        return buttons
+
+    def run(self, button):
+        if button == 'Cancel':
+            return
+
+        if button == "Refresh":
+            self.dependencies.touch()
+            return self.get_result(refresh=True, close=False)
+
+        shot_name = self._shot.name()
+        sequence_name = self._sequence.name()
+
+        # Store dependencies file paths for Blender script building
+        self._shot_data = {}
+
+        # Configure comp file
+        anim_path = self.latest_revision.get_path()
+
+        # Create empty file
+        render_path = self._ensure_file(
+            name='render',
+            format='blend',
+            to_edit=True,
+            src_path=anim_path,
+        )
+
+        # Configure jsx file
+        jsx_path = self._ensure_file(
+            name='compositing',
+            format='jsx',
+            src_path=resources.get("file_templates", "template.jsx"),
+            publish_comment="Export from Blender"
+        )
+
+        # Configure AE file
+        aep_path = self._ensure_file(
+            name='compositing',
+            format='aep',
+            to_edit=True,
+            src_path=resources.get("file_templates", "template.aep"),
+        )
+
+        # Configure render dir
+        self._ensure_folder(
+            name='passes',
+            to_edit=True,
+        )
+
+        self._department.files.touch()
+
+        # Store anim and alembic output path
+        self._shot_data["render_path"] = render_path.replace("\\", "/")
+        self._shot_data["jsx_path"] = jsx_path.replace("\\", "/")
+
+        # Build
+        super(BuildRenderScene, self).run(button)
+
+
 class CompDepartment(Department):
     _short_name = flow.Param("comp")
 
+    build_render_scene = flow.Child(BuildRenderScene)
+
+
 class MiscDepartment(Department):
     _short_name = flow.Param("misc")
 
@@ -715,6 +862,106 @@ class Shots(baseflow.film.Shots):
         return Shot
 
 
+class ShotToPlayblastDepartment(flow.Object):
+    
+    _shot = flow.Parent()
+    _shots = flow.Parent(2)
+    to_render = flow.SessionParam(False).ui(editor='bool')
+    
+    def _file_mapped_name(self):
+        return self.name() + '_blend'
+    
+    def get_file(self):
+        oid = "%s/shots/%s/departments/%s/files/%s" % (
+            self._shots.sequence.oid(),
+            self._shot.name(),
+            self.name(),
+            self._file_mapped_name(),
+        )
+        
+        if not self.root().session().cmds.Flow.exists(oid):
+            return None
+        
+        file_object = self.root().get_object(oid)
+        return file_object
+    
+    def get_icon(self):
+        return ('icons.gui', self.to_render.get() and 'check' or 'check-box-empty')
+
+class ShotToPlayblast(flow.Object):
+    
+    layout = flow.Child(ShotToPlayblastDepartment)
+    blocking = flow.Child(ShotToPlayblastDepartment)
+    animation = flow.Child(ShotToPlayblastDepartment)
+
+
+class ShotsToPlayblast(flow.DynamicMap):
+    
+    sequence = flow.Parent(2)
+    
+    @classmethod
+    def mapped_type(cls):
+        return ShotToPlayblast
+    
+    def mapped_names(self, page_num=0, page_size=None):
+        return self.sequence.shots.mapped_names()
+    
+    def columns(self):
+        return ['Name', 'Layout', 'Blocking', 'Animation']
+    
+    def _fill_row_cells(self, row, item):
+        row['Name'] = item.name()
+        row['Layout'] = ''
+        row['Blocking'] = ''
+        row['Animation'] = ''
+    
+    def _fill_row_style(self, style, item, row):
+        style["Layout_icon"] = item.layout.get_icon()
+        style["Blocking_icon"] = item.blocking.get_icon()
+        style["Animation_icon"] = item.animation.get_icon()
+
+
+class RenderSequencePlayblasts(flow.Action):
+    
+    shots = flow.Child(ShotsToPlayblast).ui(
+        expanded=True,
+        # action_submenus=True,
+        items_action_submenus=True,
+    )
+    
+    def add_to_render_pool(self, file, revision_name=None, use_simplify=False):
+        if revision_name is None:
+            rev = file.get_head_revision()
+            if rev is None or rev.get_sync_status() != 'Available':
+                self.root().session().log_error(
+                    'File %s has no head revision' % file.oid()
+                )
+                return
+            
+            revision_name = rev.name()
+
+        file.render_playblast.revision_name.set(revision_name)
+        file.render_playblast.use_simplify.set(use_simplify)
+        file.render_playblast.run('Add to render pool')
+    
+    def run(self, button):
+        for shot in self.shots.mapped_items():
+            if shot.layout.to_render.get():
+                f_layout = shot.layout.get_file()
+                if f_layout is not None:
+                    self.add_to_render_pool(f_layout)
+            
+            if shot.blocking.to_render.get():
+                f_blocking = shot.blocking.get_file()
+                if f_blocking is not None:
+                    self.add_to_render_pool(f_blocking)
+            
+            if shot.animation.to_render.get():
+                f_animation = shot.animation.get_file()
+                if f_animation is not None:
+                    self.add_to_render_pool(f_animation)
+
+
 class Sequence(baseflow.film.Sequence):
 
     _film = flow.Parent(2)
@@ -732,6 +979,7 @@ class Sequence(baseflow.film.Sequence):
         editable=False,
     )
     misc = flow.Child(MiscDepartment)
+    # render_playblasts = flow.Child(RenderSequencePlayblasts)
 
     def compute_child_value(self, child_value):
         if child_value is self.kitsu_url:
