@@ -308,7 +308,8 @@ class AssetFile(flow.Object):
     def _publish_file(self, file, comment):
         file.publish_action.comment.set(comment)
         file.publish_action.upload_after_publish.set(True)
-        file.publish_action.run("Unlock")
+        file.publish_action.keep_editing.set(False)
+        file.publish_action.run("Publish")
         
         return file.get_head_revision()
     
@@ -366,7 +367,7 @@ class AssetFiles(flow.DynamicMap):
         asset_type = self._asset._asset_type.name()
         if asset_type == "sets":
             return {
-                "dsn": ["design.ai", "layers"],
+                "dsn": ["design.ai", "layers", "layers_add"],
             }
         else:
             return {
